from .api import make_request


def send_transfer(data):
    """
    Sends a transfer using the API.
    Parameters:
    - data: A dictionary containing the transfer data.
    Returns:
    - A JSON response containing the result of the transfer.
    Raises:
    - Exception: If there is an error while sending the transfer.

    Handle sending of transfers.
    """
    url = "/transfer"
    try:
        response = make_request(url, data=data, method="POST")
        return response.json()
    except Exception as e:
        raise Exception(f"Failed to send transfer: {str(e)}")


def verify_transfer(transfer_id):
    """
    Fetches a transaction to validate/verify its status or recall its details.
    Args:
        transfer_id (str): The ID of the transfer to be verified.
    Returns:
        dict: A dictionary containing the response from the API call.
    Raises:
        Exception: If the verification of the transfer fails.

    """
    url = f"/transaction/{transfer_id}"
    try:
        response = make_request(url, method="GET")
        return response.json()
    except Exception as e:
        raise Exception(f"Failed to verify transfer: {str(e)}")


def get_transaction_history(transaction_id):
    """
    Fetches a transaction to validate/verify its status or recall its details.
    Args:
        transfer_id (str): The ID of the transfer to be verified.
    Returns:
        dict: A dictionary containing the response from the API call.
    Raises:
        Exception: If the verification of the transfer fails.

    """
    url = f"/transaction/{transaction_id}"
    try:
        response = make_request(url, method="GET")
        return response.json()
    except Exception as e:
        raise Exception(f"Failed to fetch transaction: {str(e)}")


def estimate_fees_and_speed(source, destination, amount):
    """
    Args:
        source (str): Id of the sending country.
        destination (str): Id of the receiver's country.
        amount (float): The amount to be sent.
    Returns:
        dict: A dictionary containing the estimated fees and speed of the transfer.
    Raises:
        Exception: If there is an error while estimating fees and speed.

    """
    url = f"/rates/{source}/{destination}/{amount}"
    try:
        response = make_request(url, method="GET")
        return response.json()
    except Exception as e:
        raise Exception(f"Failed to estimate fees and speed: {str(e)}")



import requests
from .settings import get_base_url
import datetime
from decouple import config

auth_token = None
expiry_time = None
user_id = config.get("SETTLE_ID")
password = config.get("SETTLE_PASSWORD")
settle_mode = config.get("SETTLE_MODE", default="live")
base_url = get_base_url()


def is_token_expired(expiry):
    expiry_date = datetime.strptime(expiry, "%d/%m/%Y")
    return datetime.now() >= expiry_date


def set_credentials(user_id, password):
    global auth_token, expiry_time
    # Simulate an API call to get new credentials
    response = requests.post(base_url, data={"user_id": user_id, "password": password})
    if response.status_code == 200:
        auth_token = response.json()
        expiry_time = auth_token["expiry"]
    else:
        raise Exception("Authentication failed")


def get_headers():
    global auth_token, expiry_time
    if not auth_token:
        raise Exception(
            "You need to authenticate first or Authentication credentials invalid"
        )

    if is_token_expired(expiry_time):
        set_credentials(user_id, password)

    return {
        "Content-Type": "application/json",
        "id": auth_token["id"],
        "token": auth_token["token"],
        "expiry": auth_token["expiry"],
        "mode": settle_mode,
    }



from .api import make_request


def get_banks_and_mobile_money(country_id):
    """
        Fetch the list of financial institutions Settle is capable of transacting to in a specific country.

    Args:
        country_id (str): Id of the country.

    Returns:
        dict: A dictionary containing the list of financial institutions.

    Raises:
        Exception: If there is an error while fetching the financial institutions.
    """
    url = f"/fis/{country_id}"
    try:
        response = make_request(url, method="GET")
        return response.json()
    except Exception as e:
        raise Exception(f"Failed to fetch financial institutions: {str(e)}")


def get_countries():
    """
    Fetch the list of countries Settle is available for sending and receiving of transfers.
    Returns:
        dict: A dictionary containing the list of countries.
    Raises:
        Exception: If there is an error while fetching the countries.
    """
    url = "/countries"
    try:
        response = make_request(url, method="GET")
        return response.json()
    except Exception as e:
        raise Exception(f"Failed to fetch countries: {str(e)}")


def resolve_account_name(fi, account_number):
    """
    Fetch the receiver's account name by supplying their account number.
    Args:
        fi (str): The financial institution ID of the receiver.
        account_number (str): The account number/username of the receiver on the financial institution.
    Returns:
        dict: A dictionary containing the receiver's account name.
    Raises:
        Exception: If there is an error while fetching the account name.
    """
    url = f"/fisit/{fi}/{account_number}"
    try:
        response = make_request(url, method="GET")
        return response.json()
    except Exception as e:
        raise Exception(f"Failed to fetch account name: {str(e)}")


def get_exchange_rates(source, destination):
    """
    Get current exchange rates between two currencies of countries within Africa.

    Args:
        source (str): Id of the sending country.
        destination (str): Id of the receiver's country.

    Returns:
        dict: A dictionary containing the current exchange rates.

    Raises:
        Exception: If there is an error while fetching the exchange rates.
    """
    url = f"/rates/{source}/{destination}"
    try:
        response = make_request(url, method="GET")
        return response.json()
    except Exception as e:
        raise Exception(f"Failed to fetch exchange rates: {str(e)}")



from .auth import set_credentials, get_headers
from .api import make_request
from .settings import get_base_url
from .transfers import (
    send_transfer,
    verify_transfer,
    get_transaction_history,
    estimate_fees_and_speed,
)
from .data_details import (
    get_banks_and_mobile_money,
    get_countries,
    get_exchange_rates,
    resolve_account_name,
)

__all__ = [
    "set_credentials",
    "make_request",
    "get_headers",
    "get_base_url",
    "send_transfer",
    "verify_transfer",
    "get_banks_and_mobile_money",
    "estimate_fees_and_speed",
    "resolve_account_name",
    "get_countries",
    "get_exchange_rates",
    "get_transaction_history",
]



import requests
from .auth import get_auth_headers
from .settings import get_env_var

def make_request(endpoint: str, data: dict = None):
    base_url = get_env_var('BASE_URL')
    headers = get_auth_headers()
    response = requests.post(f"{base_url}/{endpoint}", json=data, headers=headers)
    response.raise_for_status()
    return response.json()



from setuptools import setup, find_packages

setup(
    name="settle.africa",
    version="0.1.0",
    packages=find_packages(),
    install_requires=["requests", "python-dotenv"],
    test_suite="tests",
    author="AbdulHaleem Nasredeen",
    author_email="sales@settle.africa",
    description="A Python package for interacting with The Settle API",
    url="https://github.com/insolify/settle/python-package",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
